/****************************************************************************/
/*    			       Power Part v1.0                              */
/*		      Partitioning software for TUOX OS			    */
/*    Written by Karel Miarka & Petr Pomkla, Technical University Ostrava   */
/*         Copyright (c)1997-98 Karel Miarka, Petr Pomkla. Nov 18 1997	    */
/****************************************************************************/
#include <bios.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include <conio.h>

#define TUOX_ID 0x30 /* Tuox ID */

/****************************************************************************/
/*				Partition Item Structure		    */
/****************************************************************************/
typedef struct{
  unsigned char Active;/* Partition is Active */
  unsigned char FirstHead;/* First head */
  unsigned short FirstSectorTrack;/* First Sector and Track */
  unsigned char OSId;/* Operating system ID */
  unsigned char LastHead;/* Last Head */
  unsigned short LastSectorTrack;/* Last sector and track */
  unsigned long RelativeMove;/* Offset from begining */
  unsigned long TotalSectors;/* Number of sectors */
  unsigned char Drive;/* hda = 0x80, hdb = 0x81 ... */
} PartInfo;
/****************************************************************************/
/*			Converts Partition ID no. to String	    	    */
/****************************************************************************/
char *IDtoText(int OSID){
  switch(OSID){
    case 0x00:return "Empty";
    case 0x01:return "DOS 12-bit FAT";
    case 0x02:return "XENIX root";
    case 0x03:return "XENIX usr";
    case 0x04:return "DOS 16-bit <32M";
    case 0x05:return "Extended";
    case 0x06:return "DOS 16-bit >=32M";
    case 0x07:return "OS/2 HPFS";
    case 0x08:return "AIX";
    case 0x09:return "AIX Bootable";
    case 0x0a:return "OS/2 Boot manager";
    case 0x0b:return "FAT32";
    case TUOX_ID:return "TUOX native";
    case 0x40:return "VENIX 80286";
    case 0x51:return "NOVELL?";
    case 0x52:return "MICROPORT";
    case 0x64:return "NOVELL NETWARE";
    case 0x65:return "NOVELL NETWARE";
    case 0x75:return "PC/IX";
    case 0x80:return "Old MINIX";
    case 0x81:return "LINUX/MINIX";
    case 0x82:return "LINUX swap";
    case 0x83:return "LINUX native";
    case 0x93:return "AMOBEA";
    case 0x94:return "AMOBEA BBT";
    case 0xa5:return "BSD/386";
    case 0xb7:return "BSDI fs";
    case 0xb8:return "BSDI swap";
    case 0xc7:return "SYRINX";
    case 0xdb:return "CP/M";
    case 0xe1:return "DOS access";
    case 0xe3:return "DOS r/o";
    case 0xf2:return "DOS secondary";
    case 0xff:return "BBT";
  }
 return "Unknown";
}
/****************************************************************************/
/*			Displays Known Partitions		    	    */
/****************************************************************************/
void PartitionTypes(void){
  printf("0 Empty              9 AIX Bootable      75 PC/IX             b7 BSDI fs\n");
  printf("1 DOS 12-bit FAT     a OS/2 Boot manager 80 Old MINIX         b8 BSDI swap\n");
  printf("2 XENIX root         b FAT32             81 LINUX/MINIX       c7 SYRINX\n");
  printf("3 XENIX usr         30 TUOX native       82 LINUX swap        db CP/M\n");
  printf("4 DOS 16-bit <32M   40 VENIX 80286       83 LINUX native      e1 DOS access\n");
  printf("5 Extended          51 NOVELL?           93 AMOBEA            e3 DOS r/o\n");
  printf("6 DOS 16-bit >=32M  52 MICROPORT         94 AMOBEA BBT        f2 DOS secondary\n");
  printf("7 OS/2 HPFS         64 NOVELL NETWARE    a5 BSD/386           ff BBT\n");
  printf("8 AIX               65 NOVELL NETWARE\n");
}
/****************************************************************************/
/*			Displays Main Menu			    	    */
/****************************************************************************/
void MainMenu(void){
 printf("a  toggle a bootlable flag\n");
 printf("l  list known partition types\n");
 printf("m  print this menu\n");
 printf("p  print the partition table\n");
 printf("q  quit without saving changes\n");
 printf("t  change a partition's system ID\n");
 printf("w  write table to disk and exit\n");
}
/****************************************************************************/
/*			Calculate Track no. from SectorTrack    	    */
/****************************************************************************/
unsigned short CalcTrack(unsigned short SectorTrack){
  return(SectorTrack>>8)|((SectorTrack&0xc0)<<2);
}/* Returns Track # */
/****************************************************************************/
/*			Check if Partition's Item is Valid	    	    */
/****************************************************************************/
int Valid(unsigned char *p2Buff){
  int i;
    for(i=0;i<16;i++)
      if(p2Buff[i]) return 1;
  return 0;
}
/****************************************************************************/
/*				Yes/No Dialog				    */
/****************************************************************************/
int Yn(char *Prompt){
 int Choice;
   do{
     printf("%s",Prompt);
     Choice=tolower(getchar());fflush(stdin);
     switch(Choice){
       case 'y':return 1;
       case 'n':return 0;
     }
   }while(1);
}
/****************************************************************************/
/*  	 		  Displays all detected partitions 		    */
/****************************************************************************/
void PrintPartition(PartInfo *Pt,int N){
 int i,Pn=1,Drive;
   Drive=Pt[0].Drive;
   printf("1 Block = 1024 Bytes\n\n");
   printf("Device\tBoot\tStart\tEnd\tBlocks\tId\tSystem\n");
   for(i=0;i<N;i++){/* For all detected partitions */
     if(Drive!=Pt[i].Drive){Drive=Pt[i].Drive;Pn=1;}/* Reset Part. counter */
     printf("hd%c%d\t%c\t%d\t%d\t%lu\t%x\t%s\n",Pt[i].Drive-0x80+'a',
	     Pn++,Pt[i].Active?'*':' ',CalcTrack(Pt[i].FirstSectorTrack),
	     CalcTrack(Pt[i].LastSectorTrack),Pt[i].TotalSectors/2,
	     Pt[i].OSId,IDtoText(Pt[i].OSId));/* Print partition info */
   }
}
/****************************************************************************/
/*  	 	   Scans all drives in your system or write changes 	    */
/****************************************************************************/
int ReadWriteDrives(int *CurrPart,PartInfo *AllPart,int Write){
  unsigned char MBR[512],Temp[512],FHead;
  int CurrDrive,CurrMBROff,i,ff;
  unsigned short FSectorTrack;/* Check all drives hda .. hdd */
    for (*CurrPart=0x0,CurrDrive=0x80;CurrDrive<0x84;CurrDrive++){
      /* Scan Drive 'CurrDrive' */
      MBR[0x1fe]=0x0;/* Make Record Invalid */
      ff=biosdisk(_DISK_READ,CurrDrive,0,0,1,1,MBR);/* Read MBR */
      if(ff||(*(unsigned short*)&MBR[0x1fe]!=0xAA55)) continue;/* Success ? */
      for (i=0,CurrMBROff=0x1be;i<4;i++,CurrMBROff+=16){/*For all prim. part*/
	if(Valid(&MBR[CurrMBROff])){/* Is Current Partition Item Valid ? */
	  AllPart[*CurrPart].Drive=CurrDrive;/* Set drive */
	  if(Write) memcpy(&MBR[CurrMBROff],&AllPart[(*CurrPart)++],16);
	    else memcpy(&AllPart[(*CurrPart)++],&MBR[CurrMBROff],16);
	  if(MBR[CurrMBROff+0x4]==0x5){/* Is it extended partition ? */
	    Temp[0x1fe]=0x0;FHead=AllPart[(*CurrPart)-1].FirstHead;
	    FSectorTrack=AllPart[(*CurrPart)-1].FirstSectorTrack;
	    ff=biosdisk(_DISK_READ,CurrDrive,FHead,CalcTrack(FSectorTrack),
			FSectorTrack&0x3f,1,Temp);/* Read Extened Partition */
	    if(ff||(*(unsigned short*)&Temp[0x1fe]!=0xAA55)){/* Success ? */
	      printf("Invalid Extended Record !\n");return -1;}
	    do{
	      AllPart[*CurrPart].Drive=CurrDrive;/* Set drive */
	      if(Write){memcpy(&Temp[0x1be],&AllPart[(*CurrPart)++],16);
		ff=biosdisk(_DISK_WRITE,CurrDrive,FHead,/* Write changes */
			    CalcTrack(FSectorTrack),FSectorTrack&0x3f,1,Temp);
		if(ff||(*(unsigned short*)&Temp[0x1fe]!=0xAA55)){/*Success ?*/
		  printf("Unable to update Extended Record !\n");return -1;}
	      }	else memcpy(&AllPart[(*CurrPart)++],&Temp[0x1be],16);
	      if(Valid(&Temp[0x1ce])){/* Is Extended partition item valid ? */
		Temp[0x1fe]=0x0;FHead=Temp[0x1ce+1];/* Make buffer invalid */
		FSectorTrack=*(unsigned short*)&Temp[0x1ce+2];
		ff=biosdisk(_DISK_READ,CurrDrive,FHead,CalcTrack(FSectorTrack)
			    ,FSectorTrack&0x3f,1,Temp);/*Read next ext. part*/
		if(ff||(*(unsigned short*)&Temp[0x1fe]!=0xAA55)){/*Success ?*/
		  printf("Invalid Extended Record !\n");return -1;}
	      }else break;
	    }while (1);
	  }
	} else break;
      }
      if(Write){ff=biosdisk(_DISK_WRITE,CurrDrive,0,0,1,1,MBR);/* Update MBR*/
	if(ff||(*(unsigned short*)&MBR[0x1fe]!=0xAA55)){/* Success ? */
	  printf("Unable to update Master Boot Record !\n");return -1;}
      }
    }
 return 0;/* Let's go home ! */
}
/****************************************************************************/
/*  	 	   			Main  				    */
/****************************************************************************/
int main(void){
  PartInfo AllPart[100];
  char EditLine[100],*EndPtr;
  int CurrPart,i;
  unsigned short Sp,OsId;
  static unsigned long ResetAddress=0xFFFF0L;
    if(ReadWriteDrives(&CurrPart,AllPart,0)) return 0;/* Scan all drives */
    printf("Power Part v1.0, Partitioning software for TUOX OS\n");
    printf("Written by Karel Miarka & Petr Pomkla, Technical University Ostrava\n");
    printf("Copyright (c)1997-98 Karel Miarka, Petr Pomkla. Nov 18 1997\n");
    do{
      printf("\nCommand (m for help): ");
      i=tolower(getchar());fflush(stdin);/* Wait for key */
      printf("\n");
      switch(i){
	case 'a':/* Toggle a bootlable flag */
	  do{printf("Partition number (1-%d): ",CurrPart);/* Select Part. */
	    scanf("%s",EditLine);Sp=strtol(EditLine,&EndPtr,10);
	    if((Sp<1)||(Sp>CurrPart)||*EndPtr) continue;
	    if(AllPart[Sp-1].Active) AllPart[Sp-1].Active=0x0;
	      else AllPart[Sp-1].Active=0x80;
	    fflush(stdin);
	    break;
	  }while(1);
	break;
	case 'l':PartitionTypes();break;/* List known partition types */
	case 'm':MainMenu();break; /* Print main menu */
	case 'p':PrintPartition(AllPart,CurrPart);break;/* Print Part. Tab. */
	case 'q':return 0;/* Quit without saving changes */
	case 't':/* Change a partition's system ID */
	  do{printf("Partition number (1-%d): ",CurrPart);/* Select Part. */
	    scanf("%s",EditLine);Sp=strtol(EditLine,&EndPtr,10);
	    if((Sp>=1)&&(Sp<=CurrPart)&&!*EndPtr) break;
	  }while(1);
	  do{printf("Partition's system ID (0-ff): ",CurrPart);/* Select ID */
	    scanf("%s",EditLine);OsId=strtol(EditLine,&EndPtr,16);
	    if(!*EndPtr) break;
	  }while(1);
	  AllPart[Sp-1].OSId=OsId;
	  fflush(stdin);
	break;
	case 'w':/* Write table to disk and exit */
	  if(!Yn("Are you sure to write changes (y/n)?: ")) break;
	  if(!Yn("Are you REALY sure to write changes (y/n)?: ")) break;
	  ReadWriteDrives(&CurrPart,AllPart,1);/* Write changes */
	  printf("\nChanges have been written. Press any key to reboot ...");
	  getch();/* Waits for any key */
	  asm{/* Jump to 0xFFFF0 (Reboot) */
	    cli;
	    jmp ds:dword ptr ResetAddress;
	  }
      }
    }while(1);
}
/* End of file PPART.C */